import struct
from pathlib import Path

import numpy as np

from .specs import header_spec, dtype_spec


dtype_spec_rev = {k: v for v, k in dtype_spec.items()}


def write(path, data, header_params={}, overwrite=False):
    """
    write data to an .em file
    header_params: a dict of header parameters as defined by spec.py
    """
    path = Path(path).resolve().absolute()
    if path.exists() and not overwrite:
        raise ValueError(f'file {path} exists')
    # prepare header
    header = {
        'machine': 6,
        'version': 0,
        'unused': 0,
        'dtype': dtype_spec_rev[data.dtype.char],
        'xdim': data.shape[2],
        'ydim': data.shape[1],
        'zdim': data.shape[0],
    }
    header.update(header_params)

    for key, form in header_spec.items():
        s = struct.Struct(form)
        if form in 'ib':
            value = int(header.get(key, 0))
        else:
            value = header.get(key, 'A' * s.size).encode()
        header[key] = s.pack(value)

    # prepare data
    header_bin = b''.join(header.values())
    # TODO: cannot write dask array to bytes yet, so we have to convert
    data_bin = np.asarray(data).tobytes()

    with open(path, 'bw+') as f:
        f.write(header_bin + data_bin)



import struct
import binascii

import numpy as np

from .specs import header_spec, dtype_spec


header_struct = struct.Struct(''.join(header_spec.values()))


def read(path, header_only=False, mmap=False):
    """
    read an em file and return header info as a dict and data as a np.ndarray
    """
    with open(path, 'rb') as f:
        buffer = f.read(header_struct.size)
        header = {key: value for key, value in zip(header_spec.keys(),
                                                   header_struct.unpack(buffer))}

        # decode strings
        for k, v in header.items():
            if isinstance(v, bytes):
                header[k] = binascii.b2a_base64(v)

        data = None
        if not header_only:
            # get dtype and shape from header
            dtype = dtype_spec[header['dtype']]
            shape = header['zdim'], header['ydim'], header['xdim']

            # load data (directly from remaining data in the stream)
            if mmap:
                # must pass offset here because memmap default from beginning of file
                data = np.memmap(f, dtype=dtype, shape=shape,
                                 offset=header_struct.size, mode='r')
            else:
                data = np.fromfile(f, dtype=dtype).reshape(shape)

    return header, data
